@@ -47,19 +47,16 @@ gog_tool_select_object_point (GogView *view, double x, double y, GogObject **gob
 static void
 gog_tool_select_object_render (GogView *view)
 {
-	ArtVpath *path;
 	GogViewAllocation rect = view->allocation;
 	GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (view->model));
 	double line_width = gog_renderer_line_size (view->renderer, style->line.width);
-	
+
 	rect.x -= line_width / 2.0;
 	rect.y -= line_width / 2.0;
 	rect.w += line_width;
 	rect.h += line_width;
 
-	path = gog_renderer_get_rectangle_vpath (&rect);
-	gog_renderer_draw_sharp_path (view->renderer, path);
-	art_free (path);
+	gog_renderer_draw_selection_rectangle (view->renderer, &rect);
 }
 
 static GogTool gog_tool_select_object = {
@@ -738,6 +735,12 @@ gog_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
 	view->allocation_valid = view->child_allocations_valid = TRUE;
 }
 
+/**
+ * gog_view_update_sizes:
+ * @view : #GogView
+ *
+ * Returns: %TRUE if a redraw is necessary.
+ **/
 gboolean
 gog_view_update_sizes (GogView *view)
 {
@@ -768,13 +771,16 @@ gog_view_render	(GogView *view, GogViewAllocation const *bbox)
 	g_return_if_fail (view->renderer != NULL);
 
 	/* In particular this is true for NaNs.  */
-	if (!(view->allocation.w >= 0 && view->allocation.h >= 0))
+	if (view->model->invisible ||
+	    !(view->allocation.w >= 0 && view->allocation.h >= 0))
 		return;
 
 	if (klass->clip) {
-		gog_renderer_push_clip (view->renderer, 
-			gog_renderer_get_rectangle_vpath (&view->allocation));
+		gog_renderer_push_clip_rectangle (view->renderer, view->allocation.x, view->allocation.y,
+						  view->allocation.w, view->allocation.h);
+
 		klass->render (view, bbox);
+
 		gog_renderer_pop_clip (view->renderer);
 	}
 	else
