@@ -29,7 +29,7 @@
 
 #include <goffice/goffice-config.h>
 #include "go-color-palette.h"
-#include "goffice-gtk.h"
+#include <goffice/gtk/goffice-gtk.h>
 #include <goffice/utils/go-marshalers.h>
 
 #include <goffice/utils/go-color.h>
@@ -41,7 +41,6 @@
 
 #include <string.h>
 
-typedef struct _ColorNamePair ColorNamePair;
 struct _GOColorPalette {
 	GtkVBox	base;
 
@@ -53,10 +52,9 @@ struct _GOColorPalette {
 
 	/* only for custom colours */
 	GtkWidget   *swatches [GO_COLOR_GROUP_HISTORY_SIZE];
-	GtkTooltips *tip;
 
 	/* The table with our default color names */
-	ColorNamePair const *default_set;
+	GONamedColor const *default_set;
 };
 
 typedef struct {
@@ -77,12 +75,7 @@ enum {
 	LAST_SIGNAL
 };
 
-struct _ColorNamePair {
-	GOColor	color;
-	char const *name;	/* english name - eg. "white" */
-};
-
-static ColorNamePair const default_color_set [] = {
+GONamedColor const default_color_set [] = {
 	{ RGBA_TO_UINT (0x00, 0x00, 0x00, 0xff), N_("black")},
 	{ RGBA_TO_UINT (0x99, 0x33, 0x00, 0xff), N_("light brown")},
 	{ RGBA_TO_UINT (0x33, 0x33, 0x00, 0xff), N_("brown gold")},
@@ -203,11 +196,6 @@ go_color_palette_finalize (GObject *object)
 {
 	GOColorPalette *pal = GO_COLOR_PALETTE (object);
 
-	if (pal->tip) {
-		g_object_unref (pal->tip);
-		pal->tip = NULL;
-	}
-
 	go_color_palette_set_group (pal, NULL);
 
 	(*go_color_palette_parent_class->finalize) (object);
@@ -249,7 +237,7 @@ GSF_CLASS (GOColorPalette, go_color_palette,
  * Utility function
  */
 static gboolean
-color_in_palette (ColorNamePair const *set, GOColor color)
+color_in_palette (GONamedColor const *set, GOColor color)
 {
 	int i;
 
@@ -317,7 +305,7 @@ swatch_activated (GOColorPalette *pal, GtkBin *button)
 static gboolean
 cb_swatch_release_event (GtkBin *button, GdkEventButton *event, GOColorPalette *pal)
 {
-#warning TODO do I want to check for which button ?
+/* FIXME FIXME FIXME TODO do I want to check for which button ? */
 	swatch_activated (pal, button);
 	return TRUE;
 }
@@ -340,8 +328,9 @@ cb_swatch_key_press (GtkBin *button, GdkEventKey *event, GOColorPalette *pal)
  * Utility function
  */
 static GtkWidget *
-go_color_palette_button_new (GOColorPalette *pal, GtkTable* table, GtkTooltips *tip,
-			  ColorNamePair const * color_name, gint col, gint row)
+go_color_palette_button_new (GOColorPalette *pal, GtkTable* table,
+			     GONamedColor const *color_name,
+			     gint col, gint row)
 {
         GtkWidget *button, *swatch, *box;
 	GdkColor   gdk;
@@ -359,7 +348,7 @@ go_color_palette_button_new (GOColorPalette *pal, GtkTable* table, GtkTooltips *
 	button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_container_add (GTK_CONTAINER (button), box);
-	gtk_tooltips_set_tip (tip, button, _(color_name->name), "");
+	go_widget_set_tooltip_text (button, _(color_name->name));
 
 	gtk_table_attach (table, button, col, col+1, row, row+1,
 		GTK_FILL, GTK_FILL, 0, 0);
@@ -427,10 +416,9 @@ static GtkWidget *
 go_color_palette_setup (GOColorPalette *pal,
 		     char const *no_color_label,
 		     int cols, int rows,
-		     ColorNamePair const *color_names)
+		     GONamedColor const *color_names)
 {
 	GtkWidget	*w, *table;
-	GtkTooltips	*tip;
 	int pos, row, col = 0;
 
 	table = gtk_table_new (cols, rows, FALSE);
@@ -444,21 +432,13 @@ go_color_palette_setup (GOColorPalette *pal,
 			G_CALLBACK (cb_default_release_event), pal);
 	}
 
-	pal->tip = tip = gtk_tooltips_new ();
-#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
-	g_object_ref_sink (pal->tip);
-#else
-	g_object_ref (pal->tip);
-	gtk_object_sink (GTK_OBJECT (pal->tip));
-#endif
-
 	for (row = 0; row < rows; row++)
 		for (col = 0; col < cols; col++) {
 			pos = row * cols + col;
 			if (color_names [pos].name == NULL)
 				goto custom_colors;
 			go_color_palette_button_new ( pal,
-				GTK_TABLE (table), GTK_TOOLTIPS (tip),
+				GTK_TABLE (table),
 				&(color_names [pos]), col, row + 1);
 		}
 
@@ -466,10 +446,10 @@ custom_colors :
 	if (col > 0)
 		row++;
 	for (col = 0; col < cols && col < GO_COLOR_GROUP_HISTORY_SIZE; col++) {
-		ColorNamePair color_name = { 0, N_("custom") };
+		GONamedColor color_name = { 0, N_("custom") };
 		color_name.color = pal->group->history [col];
 		pal->swatches [col] = go_color_palette_button_new (pal,
-			GTK_TABLE (table), GTK_TOOLTIPS (tip),
+			GTK_TABLE (table),
 			&color_name, col, row + 1);
 	}
 
@@ -524,7 +504,7 @@ go_color_palette_new (char const *no_color_label,
 	GOColorPalette *pal;
 	int const cols = 8;
 	int const rows = 6;
-	ColorNamePair const *color_names = default_color_set;
+	GONamedColor const *color_names = default_color_set;
 
 	pal = g_object_new (GO_COLOR_PALETTE_TYPE, NULL);
 
@@ -644,9 +624,9 @@ cb_menu_custom_activate (GtkWidget *button, GOMenuColor *menu)
 
 /**
  * go_color_palette_make_menu:
- * @no_color_labe :
+ * @no_color_label: color label
  * default_color: #GOColor
- * @cg : #GOColorGroup
+ * @color_group: #GOColorGroup
  * @custom_dialog_title :
  * @current_color : #GOColor
  *
@@ -663,7 +643,7 @@ go_color_palette_make_menu (char const *no_color_label,
 	int cols = 8;
 	int rows = 6;
 	int col, row, pos, table_row = 0;
-	ColorNamePair const *color_names = default_color_set;
+	GONamedColor const *color_names = default_color_set;
         GtkWidget *w, *submenu;
 
 	submenu = g_object_new (go_menu_color_get_type (), NULL);
