@@ -24,6 +24,7 @@
 #include <goffice/app/go-cmd-context.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/goffice-priv.h>
+#include <goffice/gtk/go-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <atk/atkrelation.h>
@@ -204,30 +205,6 @@ go_gtk_editable_enters (GtkWindow *window, GtkWidget *w)
 		G_CALLBACK (gtk_window_activate_default), window);
 }
 
-GdkPixbuf *
-go_pixbuf_intelligent_scale (GdkPixbuf *buf, guint width, guint height)
-{
-	GdkPixbuf *scaled;
-	int w, h;
-	unsigned long int ow = gdk_pixbuf_get_width (buf);
-	unsigned long int oh = gdk_pixbuf_get_height (buf);
-
-	if (ow > width || oh > height) {
-		if (ow * height > oh * width) {
-			w = width;
-			h = width * (((double)oh)/(double)ow);
-		} else {
-			h = height;
-			w = height * (((double)ow)/(double)oh);
-		}
-
-		scaled = gdk_pixbuf_scale_simple (buf, w, h, GDK_INTERP_BILINEAR);
-	} else
-		scaled = g_object_ref (buf);
-
-	return scaled;
-}
-
 void
 go_gtk_widget_disable_focus (GtkWidget *w)
 {
@@ -692,7 +669,7 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			g_free (new_uri);
 			g_free (uri);
 			uri = NULL;
-			goto out;
+			goto loop;
 		} else {
 			g_free (uri);
 			uri = new_uri;
@@ -1014,134 +991,49 @@ go_gtk_query_yes_no (GtkWindow *parent, gboolean default_answer,
 		go_gtk_dialog_run (GTK_DIALOG (dialog), parent);
 }
 
-/**
- * go_pixbuf_new_from_file :
- * @filename :
- *
- * utility routine to create pixbufs from file @name in the goffice_icon_dir.
- *
- * Returns: a GdkPixbuf that the caller is responsible for.
- **/
-GdkPixbuf *
-go_pixbuf_new_from_file (char const *filename)
+#ifndef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
+static GtkTooltips *
+get_tooltips (GObject *obj)
 {
-	char *path = g_build_filename (go_sys_icon_dir (), filename, NULL);
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	g_free (path);
-	return pixbuf;
-}
+	GtkTooltips *tips = g_object_get_data (obj, "-go-tips");
 
-#ifndef HAVE_GDK_CAIRO_SET_SOURCE_PIXBUF 
-/* This is public from 2.8 onwards.   */
-static void
-gdk_cairo_set_source_pixbuf (cairo_t   *cr,
-			     GdkPixbuf *pixbuf,
-			     double     pixbuf_x,
-			     double     pixbuf_y)
-{
-  gint width = gdk_pixbuf_get_width (pixbuf);
-  gint height = gdk_pixbuf_get_height (pixbuf);
-  guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
-  int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-  guchar *cairo_pixels;
-  cairo_format_t format;
-  cairo_surface_t *surface;
-  static const cairo_user_data_key_t key;
-  int j;
+	if (!tips) {
+		tips = gtk_tooltips_new ();
 
-  if (n_channels == 3)
-    format = CAIRO_FORMAT_RGB24;
-  else
-    format = CAIRO_FORMAT_ARGB32;
-
-  cairo_pixels = g_malloc (4 * width * height);
-  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
-						 format,
-						 width, height, 4 * width);
-  cairo_surface_set_user_data (surface, &key,
-			       cairo_pixels, (cairo_destroy_func_t)g_free);
-
-  for (j = height; j; j--)
-    {
-      guchar *p = gdk_pixels;
-      guchar *q = cairo_pixels;
-
-      if (n_channels == 3)
-	{
-	  guchar *end = p + 3 * width;
-	  
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      q[0] = p[2];
-	      q[1] = p[1];
-	      q[2] = p[0];
-#else	  
-	      q[1] = p[0];
-	      q[2] = p[1];
-	      q[3] = p[2];
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+		g_object_ref_sink (tips);
+#else
+		g_object_ref (tips);
+		gtk_object_sink (GTK_OBJECT (tips));
 #endif
-	      p += 3;
-	      q += 4;
-	    }
-	}
-      else
-	{
-	  guchar *end = p + 4 * width;
-	  guint t1,t2,t3;
-	    
-#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
-
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      MULT(q[0], p[2], p[3], t1);
-	      MULT(q[1], p[1], p[3], t2);
-	      MULT(q[2], p[0], p[3], t3);
-	      q[3] = p[3];
-#else	  
-	      q[0] = p[3];
-	      MULT(q[1], p[0], p[3], t1);
-	      MULT(q[2], p[1], p[3], t2);
-	      MULT(q[3], p[2], p[3], t3);
-#endif
-	      
-	      p += 4;
-	      q += 4;
-	    }
-	  
-#undef MULT
+		g_object_set_data_full (obj, "-go-tips", tips,
+					(GDestroyNotify)g_object_unref);
 	}
 
-      gdk_pixels += gdk_rowstride;
-      cairo_pixels += 4 * width;
-    }
-
-  cairo_set_source_surface (cr, surface, pixbuf_x, pixbuf_y);
-  cairo_surface_destroy (surface);
+	return tips;
 }
 #endif
 
-/**
- * go_cairo_set_source_pixbuf:
- * @cr: a #Cairo context
- * @pixbuf: a #GdkPixbuf
- * @pixbuf_x: X coordinate of location to place upper left corner of @pixbuf
- * @pixbuf_y: Y coordinate of location to place upper left corner of @pixbuf
- * 
- * Sets the given pixbuf as the source pattern for the Cairo context.
- * The pattern has an extend mode of %CAIRO_EXTEND_NONE and is aligned
- * so that the origin of @pixbuf is @pixbuf_x, @pixbuf_y
- *
- **/
 void
-go_cairo_set_source_pixbuf (cairo_t   *cr,
-			    GdkPixbuf *pixbuf,
-			    double     pixbuf_x,
-			    double     pixbuf_y)
+go_widget_set_tooltip_text (GtkWidget *widget, const gchar *tip)
 {
-	gdk_cairo_set_source_pixbuf (cr, pixbuf, pixbuf_x, pixbuf_y);
+#ifdef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
+	gtk_widget_set_tooltip_text (widget, tip);
+#else
+	GtkTooltips *tips = get_tooltips (G_OBJECT (widget));
+	gtk_tooltips_set_tip (tips, widget, tip, NULL);
+#endif
+}
+
+void
+go_tool_item_set_tooltip_text (GtkToolItem *item, const gchar *tip)
+{
+#ifdef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
+	gtk_tool_item_set_tooltip_text (item, tip);
+#else
+	GtkTooltips *tips = get_tooltips (G_OBJECT (item));
+	gtk_tool_item_set_tooltip (item, tips, tip, NULL);
+#endif
 }
 
 
@@ -1233,3 +1125,66 @@ go_dialog_guess_alternative_button_order (GtkDialog *dialog)
 								    new_order);
 	g_free (new_order);
 }
+
+/**
+ * go_menu_position_below:
+ * @menu: a #GtkMenu
+ * @x:
+ * @y:
+ * @push_in:
+ * @user_data:
+ *
+ * Implementation of a GtkMenuPositionFunc that positions
+ * the child window under the parent one, for use with gtk_menu_popup.
+ **/
+
+void
+go_menu_position_below (GtkMenu  *menu,
+			gint     *x,
+			gint     *y,
+			gint     *push_in,
+			gpointer  user_data)
+{
+	GtkWidget *widget = GTK_WIDGET (user_data);
+	gint sx, sy;
+	GtkRequisition req;
+	GdkScreen *screen;
+	gint monitor_num;
+	GdkRectangle monitor;
+
+	gdk_window_get_origin (widget->window, &sx, &sy);
+
+	if (GTK_WIDGET_NO_WINDOW (widget))
+	{
+		sx += widget->allocation.x;
+		sy += widget->allocation.y;
+	}
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &req);
+
+	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
+		*x = sx;
+	else
+		*x = sx + widget->allocation.width - req.width;
+	*y = sy;
+
+	screen = gtk_widget_get_screen (widget);
+	monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+	if (*x < monitor.x)
+		*x = monitor.x;
+	else if (*x + req.width > monitor.x + monitor.width)
+		*x = monitor.x + monitor.width - req.width;
+
+	if (monitor.y + monitor.height - *y - widget->allocation.height >= req.height)
+		*y += widget->allocation.height;
+	else if (*y - monitor.y >= req.height)
+		*y -= req.height;
+	else if (monitor.y + monitor.height - *y - widget->allocation.height > *y - monitor.y)
+		*y += widget->allocation.height;
+	else
+		*y -= req.height;
+
+	*push_in = FALSE;
+}
