@@ -1558,6 +1558,8 @@ go_format_parse_number_E (GOFormatParseState *pstate)
 
 		simplify_mantissa = TRUE;
 		for (i = 0; i < pstate->tno_E; i++) {
+			if (GET_TOKEN(i).token == '.')
+				break;
 			if (GET_TOKEN(i).token == '0') {
 				simplify_mantissa = FALSE;
 				break;
@@ -3528,8 +3530,19 @@ go_format_inc_precision (GOFormat const *fmt)
 			break;
 		}
 
+		case 'E': 
+			if (last_zero != -2) {
+				if (last_zero>=0)
+					g_string_insert_len (res, last_zero + 1, ".0",2);
+				else
+					g_string_append (res, ".0");
+				last_zero = -2;
+			}
+			g_string_append_c (res, t);
+			break;
+			
 		case '0':
-			if (last_zero == -1)
+			if (last_zero != -2)
 				last_zero = res->len;
 			/* Fall through.  */
 
