@@ -27,8 +27,8 @@
 #include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-series-lines.h>
+#include <goffice/math/go-math.h>
 #include <goffice/utils/go-format.h>
-#include <goffice/utils/go-math.h>
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <libart_lgpl/libart.h>
